def xmltodict_ensure_list(value, key):
    """Make sure response is a list

    xmltodict doesn't know schema's so doesn't know
    something should be a list or not.
    """
    if value is None:
        return []

    value = value[key]

    if isinstance(value, list):
        return value

    return [value]


def create_command(command, extra_data={}):
    """Create a basic command string

    This is used to create a command string that can be used
    to send to the device.
    """
    return dicttoxml(
        {
            "Command": {
                "Name": command,
                **extra_data,
            },
        },
    )


def dicttoxml(value):
    if isinstance(value, dict):
        return "".join(f"<{key}>{dicttoxml(val)}</{key}>" for key, val in value.items())

    if isinstance(value, list):
        return "".join(dicttoxml(val) for val in value)

    return value



"""EAGLE-200 hub."""
from __future__ import annotations

import asyncio
import logging
from time import time

from aiohttp import ClientSession, BasicAuth
import xmltodict

from .electric_meter import ElectricMeter
from .errors import BadAuth
from .util import create_command, xmltodict_ensure_list

_LOGGER = logging.getLogger(__name__)


class EagleHub:
    """EAGLE-200 hub."""

    def __init__(
        self,
        session: ClientSession,
        cloud_id: str,
        install_code: str,
        *,
        host: str | None = None,
    ) -> None:
        """Initialize hub."""
        self.session = session
        if host is None:
            self.host = f"eagle-{cloud_id}.local"
        else:
            self.host = host
        self.cloud_id = cloud_id
        self.install_code = install_code
        self.devices = []
        self.auth = BasicAuth(cloud_id, install_code)
        self.next_request = time()

    async def make_request(self, command_xml: str):
        """Make a request."""
        wait_time = self.next_request - time()
        if wait_time > 0:
            await asyncio.sleep(wait_time)

        async with self.session.post(
            f"http://{self.host}/cgi-bin/post_manager",
            auth=self.auth,
            headers={"content-type": "text/xml"},
            data=command_xml,
        ) as response:
            # Wait a second until the next request
            self.next_request = time() + 1

            if response.status == 401:
                raise BadAuth

            text = await response.text()
            return xmltodict.parse(text, dict_constructor=dict)

    async def get_device_list(self) -> list[ElectricMeter]:
        """Get a list of devices."""
        response = await self.make_request(create_command("device_list"))
        result = []

        for device in xmltodict_ensure_list(response["DeviceList"], "Device"):
            if device["ModelId"] == "electric_meter":
                result.append(ElectricMeter(device, self.make_request))
            else:
                _LOGGER.debug(f"Skipping unknown device {device['ModelId']}")

        return result



from .util import xmltodict_ensure_list, create_command


class ElectricMeter:
    """Represent an electric meter.

    {'ConnectionStatus': 'Connected',
      'HardwareAddress': '0x00abcd123',
      'LastContact': '0x611d885a',
      'Manufacturer': 'Generic',
      'ModelId': 'electric_meter',
      'Name': 'Power Meter',
      'NetworkAddress': '0x0000',
      'Protocol': 'Zigbee'}
    """

    # Bug in the API is causing CurrentSummationDelivered not
    # to be returned. So just fetch all (default behavior)
    ENERGY_AND_POWER_VARIABLES = [
        "zigbee:InstantaneousDemand",
        "zigbee:CurrentSummationDelivered",
        "zigbee:CurrentSummationReceived",
    ]

    @classmethod
    def create_instance(cls, hub, hardware_address):
        """Create a new electric meter."""
        return cls(
            {
                "HardwareAddress": hardware_address,
            },
            hub.make_request,
        )

    def __init__(self, details, make_request):
        """Initialize the electric meter."""
        self.details = details
        self.make_request = make_request

    @property
    def is_connected(self) -> bool:
        """Return connection status."""
        return self.details["ConnectionStatus"] == "Connected"

    @property
    def connection_status(self) -> str:
        return self.details["ConnectionStatus"]

    @property
    def hardware_address(self) -> str:
        return self.details["HardwareAddress"]

    @property
    def last_contact(self) -> str:
        return self.details["LastContact"]

    @property
    def manufacturer(self) -> str:
        return self.details["Manufacturer"]

    @property
    def model_id(self) -> str:
        return self.details["ModelId"]

    @property
    def name(self) -> str:
        return self.details["Name"]

    @property
    def network_address(self) -> str:
        return self.details["NetworkAddress"]

    @property
    def protocol(self) -> str:
        return self.details["Protocol"]

    def create_command(self, command, extra_data={}):
        """Create command targeting this device."""
        return create_command(
            command,
            {"DeviceDetails": {"HardwareAddress": self.hardware_address}, **extra_data},
        )

    async def get_device_details(self):
        data = await self.make_request(
            self.create_command(
                "device_details",
            )
        )
        return xmltodict_ensure_list(data["Device"]["Components"], "Component")

    async def get_device_query(self, variables=None):
        """Query data."""
        if variables is None:
            components = {"All": "Y"}
        else:
            components = {
                "Component": {
                    "Name": "Main",
                    "Variables": [{"Variable": {"Name": var}} for var in variables],
                }
            }
        data = await self.make_request(
            self.create_command(
                "device_query",
                {"Components": components},
            )
        )
        self.details = data["Device"]["DeviceDetails"]

        result = {}
        for component in xmltodict_ensure_list(
            data["Device"]["Components"], "Component"
        ):
            for variable in component["Variables"]["Variable"]:
                result[variable["Name"]] = variable

        return result

    def __repr__(self) -> str:
        return f"<ElectricMeter {self.details.get('Name', '')}>"



from setuptools import setup


long_description = open("README.md").read()

setup(
    name="aioeagle",
    version="1.1.0",
    license="Apache License 2.0",
    url="https://github.com/home-assistant-libs/aioeagle",
    author="Paulus Schoutsen",
    author_email="paulus@paulusschoutsen.nl",
    description="Python module to talk to Rainforest EAGLE-200.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["aioeagle"],
    zip_safe=True,
    platforms="any",
    install_requires=list(val.strip() for val in open("requirements.txt")),
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
